/**
 * 事件分发器，用以注册及广播相应事件
 * 适用各模块间的通信及java与js的通信
 * User: qingwu
 * Date: 13-7-23
 * Time: 下午11:15
 */

(function (win) {
    var _eventDispatcher = win.eventDispatcher = new EventDispatcher() || {};

    /**
     * 事件分发类
     * @constructor
     */
    function EventDispatcher() {
        var _this = this;
        _this.events = {};

        /**
         * 注册事件
         * @param type 事件名称
         * @param listen 事件关联方法
         */
        _this.addEventListener = function (type, listen) {
            if (typeof listen !== 'function') {
                return;
            }
            var _events = _this.events;
            if (_events.hasOwnProperty(type)) {
                var index = inArray(listen, _events[type]);
                if (index < 0) {
                    _events[type].push(listen);
                }
            }
            else {
                _events[type] = [];
                _events[type].push(listen);
            }
        }

        /**
         * 移除事件
         * @param type 方法名称
         * @param listen 事件关联方法
         */
        _this.removeEventListener = function (type, listen) {
            var _events = _this.events;
            if (_events.hasOwnProperty(type)) {
                var _event = _events[type];
                if (typeof listen === 'function') {
                    var index = inArray(listen, _event);
                    if (index >= 0) {
                        _event.splice(index, 1);
                    }
                }
                else {
                    _event.length = 0;
                    delete _events[type];
                }
            }
        }

        /**
         * 触发事件执行相关方法
         * @param type
         * @param option
         */
        _this.trigger = function (type, option) {
            var _events = _this.events;
            if (_events.hasOwnProperty(type)) {
                var _event = _events[type];
                for (var eIndex = 0, eLength = _event.length; eIndex < eLength; eIndex++) {
                    var handler = _event[eIndex];
                    if (typeof handler === 'function') {
                        handler(option);
                    }
                }
            }
        }

        /**
         * 获取元素在数组中的下标
         * @param elem 需要定位的元素
         * @param array 数组
         * @returns {number}
         */
        function inArray(elem, array) {
            for (var i = 0, length = array.length; i < length; i++) {
                if (array[i] === elem) {
                    return i;
                }
            }
            return -1;
        }
    }

})(window)


